{*******************************************}
{   Statistics for MtxVec                   }
{                                           }
{   UNIT STATSERIES                         }
{                                           }
{   By Marjan Slatinek                      }
{   Copyright (c) 1999-2003 Dew Research    }
{   All Rights Reserved                     }
{*******************************************}

{:New Statistical series and Quality Control Chart routines.
 New Statistical series and Quality Control Chart routines. All routines and series styles require
 TeeChart v5 (or above) correctly installed on your machine. If you don't have TeeChart v5.0 (or
 above) installed on your machine, you can download the latest version (trial or registered) from
 Steema's site: <a href="http://www.steema.com">www.steema.com</a>.}
unit StatSeries;

interface

uses Chart, TeEngine, TeCanvas, Series, TeeConst, TeeProcs, Windows, Classes,
     Probabilities, MtxVec, Math387, Statistics, SysUtils, Graphics, MtxVecTee;

{:Plots probabilities charts.
  The TStatProbSeries inherits all TPointSeries properties/methods
  and introduces some new properies. TStatProbSeries can be used to construct "special"
  statistical plots:<br>
  <ul>
  <li>Normal probability plot,</li>
  <li>Quantile-Quantile plot,</li>
  <li>Weibull probability plot.</li>
  </ul><p>
  <u>Note:</u>TStatProbSeries does not automatically change chart vertical and/or horizontal axis
  labels. This is done by adding <See Class="TStatAxisLabelTool"> tool to chart tool list, setting it's
  properties and activating it.
  @example In the following example Weibull probabilities plot is built from "scratch" - without
  adding any series/tool to chart at design time.<p>
  <code>
  Uses MtxVec, MtxVecTee, StatSeries, StatAxisLabelTool,
    Math387;
  var Data, XVec, YVec: TVec;
    X1,Y1,X2,Y2: TSample;
  begin
    Data := TVec.Create;
    XVec := TVec.Create;
    YVec := TVec.Create;
    try
      Chart1.FreeAllSeries(nil);
      Chart1.Tools.Clear;
      Chart1.AddSeries(TStatProbSeies.Create(Chart1));
      with TStatAxisLabelTool(Chart1.Tools.Add(TStatAxisLabelTool.Create(Chart1))) do
      begin
        Series := Chart1.Series[0];
        ProbSeriesType := stWeibullSeries;
      end;
      // generate some random values for Data vec
      Data.Size(100);
      RandomWeibull(1.2,3,Data); // Random Weibull with a=1.2, b=3.0
      StatWeibullPlot(Data,XVec,YVec,MinX,MaxX,MinY,MaxY,false);
      With TStatProbSeries(Chart1.Series[0]) do
      begin
        MinX := X1;
        MinY := Y1;
        MaxX := X2;
        MaxY := Y2;
      end;
      DrawValues(XVec,YVec,Chart1.Series[0]);
    finally
      XVec.Destroy;
      YVec.Destroy;
      Data.Destroy;
    end;
  end;
  </code>
  @cat Probabilities Plots
}
type TStatProbSeries=class(TPointSeries)
  private
    { internal}
    FSlopePen: TChartPen;
    procedure SetSlopePen(const Value: TChartPen);
  protected
    procedure SetSeriesColor(AColor: TColor); override;
  public
    {:Define the slope line start point.
      Define the slope line start point (minX,minY).
      These values are calculate in  <See Routine=StatNormalPlot>, <See Routine=StatQQPlot>
      or <See Routine=StatWeibullPlot> procedure.}
    MinX,MinY: TSample;
    {:Define the slope line end point
      Define the slope line end point (maxX,maxY).
      These values are calculate in  <See Routine=StatNormalPlot>, <See Routine=StatQQPlot>
      or <See Routine=StatWeibullPlot> procedure.}
    MaxX,MaxY: TSample;
    {:@exclude}
    constructor Create(AOwner: TComponent); override;
    {:@exclude}
    destructor Destroy; override;
    {:@exclude}
    procedure DoAfterDrawValues; override;
    {:@exclude}
    procedure AddSampleValues(NumValues: Integer); override;
  published
    {:Access SlopePen propery to control slope pen properties
      (style, color, ...).
    @example  To hide slope pen, you can use the following code:<p>
    <code>
    StatSpecialSeries.SlopePen.Visible := false;
    </code>
    }
    property SlopePen : TChartPen read FSlopePen write SetSlopePen;
  end;

{:Plots Quality controls charts.
  The TQCSeries can be used to plot Quality Control Charts (variable
  and attribute charts). It inherits all TLineSeries properties/methods
  and introduces some new properties.
  @example
  <code>
  var center, upcl, lowcl: TSample;
    Data: TMtx;
    EWMAResult: TVec;
  begin
    Data := TMtx.Create;
    EWMAResult := TVec.Create;
    try
      Data.LoadFromFile('ewma_data.mtx');
      // EWMA with r= 0.37
      EWMAChart(Data,DrawVec,center,upcl,lowcl,0.37,0.95);
      // setup TQCSeries
      Series1.CL := center;
      Series1.UCL := upcl;
      Series1.LCL := lowcl;
      Series1.ShowText := True;
      DrawValues(DrawVec,Series1);
    finally
      Data.Destroy;
      EWMAResult.Destroy;
    end;
  end;
  </code>
  @cat Control Charts
}
type TQCSeries=class(TLineSeries)
  private
    FLCL: TSample;
    FUCL: TSample;
    FControlLimitPen: TChartPen;
    FCLPen: TChartPen;
    FCL: TSample;
    FShowText: boolean;
    procedure SetLCL(const Value: TSample);
    procedure SetUCL(const Value: TSample);
    procedure SetControlLimitPen(const Value: TChartPen);
    procedure SetCLPen(const Value: TChartPen);
    procedure SetCL(const Value: TSample);
    procedure SetShowText(const Value: boolean);
  protected
    procedure DoAfterDrawValues; override;
    procedure SetSeriesColor(AColor: TColor); override;
    class Function GetEditorClass:String; override;
  public
    {:@exclude}
    function MaxYValue:Double; override;
    {:@exclude}
    function MinYValue:Double; override;
    {:@exclude}
    function IsValidSourceOf(Value:TChartSeries):Boolean; override;
    {:@exclude}
    procedure Assign(Source:TPersistent);override;
    {:@exclude}
    constructor Create(AOwner: TComponent); override;
    {:@exclude}
    destructor Destroy; override;
  published
    {:Defines Quality Control Chart <b>C</b>enter<b>l</b>ine.}
    property CL: TSample read FCL write SetCL;
    {:Defines Quality Control Chart <b>L</b>ower <b>C</b>ontrol <b>L</b>imit.}
    property LCL : TSample read FLCL write SetLCL;
    {:Defines Quality Control Chart <b>U</b>pper <b>C</b>ontrol <b>L</b>imit.}
    property UCL : TSample read FUCL write SetUCL;
    {: Control limit pen.
      Access ControlLimitPen propery to control control limits pen properties
      (style, color, ...).
    @example  To set control limit color to yellow and it's width to 2, you can use
      the following code:<p>
    <code>
    StatQCSeries.ControlLimitPen.Color := clYellow;
    StatQCSeries.ControlLimitPen.Width := 2;
    </code>
    }
    property ControlLimitPen: TChartPen read FControlLimitPen write SetControlLimitPen;
    {:Center line pen.
      Access CLPen propery to control chart center line pen properties
      (style, color, ...).
    }
    property CLPen: TChartPen read FCLPen write SetCLPen;
    {:Show/Hide control limits value.
      If True, CL, LCL and UCL values are displayed next to control limit line.
    }
    property ShowText: boolean read FShowText write SetShowText default True;
  end;

{:Constructs X-Chart ("XBar Chart").
  Calculates X-Chart ("XBar Chart") drawing values, center line, upper and lower control limit.
  @param Data Each Data row contains replicated observation taken at specific time.
  @param Confidence Confidence level for upper and lower control limit. Confidence must lie in the (0,1) interval.
  @param DrawVec Returns values to be drawn.
  @param CL Returns control Chart centerline.
  @param UCL Returns control Chart upper control limit.
  @param LCL Returns control Chart lower control limit.
  @example  The following code will create XBar chart:<p>
  <code>
  var CL, UCL, LCL : TSample;
    Data: TMtx;
    DrawVec: TVec;
    Confidence: TSample;
  begin
    Data := TMtx.Create;
    DrawVec := TVec.Create;
    try
      Data.LoadFromFile('ewma_data.mtx');
      QCXChart(Data,DrawVec,CL,UCL,LCL,0.025);
      // setup limit lines
      Series1.UCL := UCL;
      Series1.LCL := LCL;
      Series1.CL := CL;
      // now draw actual data
      DrawValues(DrawVec,Series1);
    finally
      Data.Destroy;
      DrawVec.Destroy;
    end;
  end;
  </code>
  @SeeAlso <See Class="TQCSeries">
  @SeeAlso <See Routine="QCSChart">
  @SeeAlso <See Routine="QCRChart">
  @cat Control Charts
}
procedure QCXChart(Data: TMtx; DrawVec: TVec; var CL, UCL, LCL: TSample; Confidence: TSample = 0.997);

{:Constructs S-Chart.
  Calculates S-Chart drawing values, center line, upper and lower control limit.
  @param Data Each Data row contains replicated observation taken at specific time.
  @param Confidence Confidence level for upper and lower control limit. Confidence must lie in the (0,1) interval.
  @param DrawVec Returns values to be drawn.
  @param CL Returns control Chart centerline.
  @param UCL Returns control Chart upper control limit.
  @param LCL Returns control Chart lower control limit.
  @example  The following code will create S chart:<p>
  <code>
  var CL, UCL, LCL : TSample;
    Data: TMtx;
    DrawVec: TVec;
    Confidence: TSample;
  begin
    Data := TMtx.Create;
    DrawVec := TVec.Create;
    try
      Data.LoadFromFile('data.mtx');
      QCSChart(Data,DrawVec,CL,UCL,LCL,0.025);
      // setup limit lines
      Series1.UCL := UCL;
      Series1.LCL := LCL;
      Series1.CL := CL;
      // now draw actual data
      DrawValues(DrawVec,Series1);
    finally
      Data.Destroy;
      DrawVec.Destroy;
    end;
  end;
  </code>
  @SeeAlso <See Class="TQCSeries">
  @SeeAlso <See Routine="QCXChart">
  @SeeAlso <See Routine="QCRChart">
  @cat Control Charts
}
procedure QCSChart(Data: TMtx; DrawVec: TVec; var CL, UCL, LCL: TSample; Confidence: TSample = 0.997);

{:Constructs R-Chart.
  Calculates R-Chart drawing values, center line, upper and lower control limit.
  @param Data Each Data row contains replicated observation taken at specific time.
  @param Confidence Confidence level for upper and lower control limit. Confidence must lie in the (0,1) interval.
  @param DrawVec Returns values to be drawn.
  @param CL Returns control Chart centerline.
  @param UCL Returns control Chart upper control limit.
  @param LCL Returns control Chart lower control limit.
  @example  The following code will create R chart:<p>
  <code>
  var CL, UCL, LCL : TSample;
    Data: TMtx;
    DrawVec: TVec;
    Confidence: TSample;
  begin
    Data := TMtx.Create;
    DrawVec := TVec.Create;
    try
      Data.LoadFromFile('ewma_data.mtx');
      QCRChart(Data,DrawVec,CL,UCL,LCL,0.025);
      // setup limit lines
      Series1.UCL := UCL;
      Series1.LCL := LCL;
      Series1.CL := CL;
      // now draw actual data
      DrawValues(DrawVec,Series1);
    finally
      Data.Destroy;
      DrawVec.Destroy;
    end;
  end;
  </code>
  @SeeAlso <See Class="TQCSeries">
  @SeeAlso <See Routine="QCXChart">
  @SeeAlso <See Routine="QCSChart">
  @cat Control Charts
}
procedure QCRChart(Data: TMtx; DrawVec: TVec; var CL, UCL, LCL: TSample; Confidence: TSample = 0.997);

{:Calculates the Process Capability indexes p, Cp and Cpk.
  Calculates the Process Capability indexes p, Cp and Cpk.
  @param Data Process data.
  @param LB,UB Lower and upper specification limit. LB and UB are boundaries within which measurements on a product characteristic must lie.
  @param Alpha Desired significance level.
  @param p Returns calculated significance.
  @param CP Returns capability index.
  @param CPConfInt Returns 100*(1-Alpha) Cp confindence interval.
  @param CPK Returns capability index for uncentred process.
  @param CPKConfInt Returns 100*(1-Alpha) CPK confidence interval.
  @example In the following example capability indexes and their condifence
    are calculated by using QCCapIndexes routine.<p>
  <code>
  var Data: TVec;
      p, CP, CPK, Alpha: TSample;
      CPConfInt, CPKConfInt: TTwoElmReal;
  begin
    Data := TVec.Create;
    try
      Data.LoadFromFile('PCDATA.vec');
      QCCapIndexes(Data,LB,UB,p,CP,CPK,CPConfInt,CPKConfInt, Alpha);
    finally
      Data.Destroy;
    end;
  end;
  </code>
}
procedure QCCapIndexes(Data: TVec; LB, UB: TSample; var p, CP, CPK : TSample;
                       var CPConfInt, CPKConfInt: TTwoElmReal; Alpha: TSample = 0.05);

{:Constructs P-Chart.
  Calculates the P-Chart drawing values, center line, upper and lower control limits.
  The assumption is all samples have the same SampleSize.
}
procedure QCPChart(Data: TVec; SampleSize : Integer; DrawVec: TVec; var CL, UCL, LCL : TSample; Confidence: TSample = 0.997); overload;
{:@desc In this case each sample can have different size. You must store sizes in SampleSize vector.
  An exeption is raised if Data and SampleSize length do not match.
  @param Data Data to be analyzed. Each value represents number of defects.
  @param Confidence Confidence level for upper and lower control limit. Confidence must lie
    in the (0,1) interval.
  @param Samplesize Sample size. Can be integer or vector.
  @param DrawVec Returns values to be drawn.
  @param CL Returns control Chart centerline.
  @param UCL Returns control Chart upper control limit.
  @param LCL Returns control Chart lower control limit.
  @example  The following code will create P chart:<p>
  <code>
  var CL, UCL, LCL : TSample;
    Data: TMtx;
    DrawVec: TVec;
    Confidence: TSample;
  begin
    Data := TMtx.Create;
    DrawVec := TVec.Create;
    try
      Data.LoadFromFile('data.mtx');
      QCPChart(Data,DrawVec,CL,UCL,LCL,0.05);
      // setup limit lines
      Series1.UCL := UCL;
      Series1.LCL := LCL;
      Series1.CL := CL;
      // now draw actual data
      DrawValues(DrawVec,Series1);
    finally
      Data.Destroy;
      DrawVec.Destroy;
    end;
  end;
  </code>
  @SeeAlso <See Class="TQCSeries">
  @SeeAlso <See Routine="QCNPChart">
  @SeeAlso <See Routine="QCUChart">
  @SeeAlso <See Routine="QCCChart">
  @cat Control Charts
}
procedure QCPChart(Data: TVec; SampleSize : TVec; DrawVec: TVec; var CL: TSample; UCL,LCL: TVec; Confidence: TSample = 0.997); overload;

{:Constructs nP-Chart.
  Constructs nP-Chart drawing values, center line, upper and lower control limit.
  The assumption is all samples have the same SampleSize.
}
procedure QCNPChart(Data: TVec; SampleSize : Integer; DrawVec: TVec; var CL, UCL, LCL : TSample; Confidence: TSample = 0.997); overload;
{:@desc In this case each sample can have different size. You must store sizes in SampleSize vector.
  An exeption is raised if Data and SampleSize length do not match.
  @param Data Data to be analyzed. Each value represents number of defects.
  @param Confidence Confidence level for upper and lower control limit. Confidence must lie
    in the (0,1) interval.
  @param Samplesize Sample size. Can be integer or vector.
  @param DrawVec Returns values to be drawn.
  @param CL Returns control Chart centerline.
  @param UCL Returns control Chart upper control limit.
  @param LCL Returns control Chart lower control limit.
  @SeeAlso <See Class="TQCSeries">
  @SeeAlso <See Routine="QCPChart">
  @SeeAlso <See Routine="QCUChart">
  @SeeAlso <See Routine="QCCChart">
  @cat Control Charts
}
procedure QCNPChart(Data: TVec; SampleSize : TVec; DrawVec: TVec; CL, UCL,LCL: TVec; Confidence: TSample); overload;

{:Constructs U-Chart.
  Constructs U-Chart drawing values, center line, upper and lower control limit.
  The assumption is all samples have the same SampleSize.
}
procedure QCUChart(Data: TVec; SampleSize : Integer; DrawVec: TVec; var CL, UCL, LCL : TSample; Confidence: TSample = 0.997); overload;
{:@desc In this case each sample can have different size. You must store sizes in SampleSize vector.
  An exeption is raised if Data and SampleSize length do not match.
  @param Data Data to be analyzed. Each value represents number of defects.
  @param Confidence Confidence level for upper and lower control limit. Confidence must lie
    in the (0,1) interval.
  @param Samplesize Sample size. Can be integer or vector.
  @param DrawVec Returns values to be drawn.
  @param CL Returns control Chart centerline.
  @param UCL Returns control Chart upper control limit.
  @param LCL Returns control Chart lower control limit.
  @SeeAlso <See Class="TQCSeries">
  @SeeAlso <See Routine="QCPChart">
  @SeeAlso <See Routine="QCNPChart">
  @SeeAlso <See Routine="QCCChart">
  @cat Control Charts
}
procedure QCUChart(Data: TVec; SampleSize : TVec; DrawVec: TVec; var CL: TSample; UCL,LCL: TVec; Confidence: TSample = 0.997); overload;

{:Constructs C-Chart.
  Constructs C-Chart drawing values, center line, upper and lower control limit.
  @param Data Data to be analyzed. Each value represents number of defects.
  @param Confidence Confidence level for upper and lower control limit. Confidence must lie
    in the (0,1) interval.
  @param Samplesize Sample size. Can be integer or vector.
  @param DrawVec Returns values to be drawn.
  @param CL Returns control Chart centerline.
  @param UCL Returns control Chart upper control limit.
  @param LCL Returns control Chart lower control limit.
  @SeeAlso <See Class="TQCSeries">
  @SeeAlso <See Routine="QCPChart">
  @SeeAlso <See Routine="QCNPChart">
  @SeeAlso <See Routine="QCUChart">
  @cat Control Charts
}
procedure QCCChart(Data: TVec; DrawVec: TVec; var CL, UCL, LCL : TSample; Confidence: TSample);

{:Constructs EWMA Control Chart.
  Calculates the Exponential Weighted Moving Average (EWMA) control chart. In this case UCL and LCL are constant (asymptote) limits.
  This chart is also known as exponentially smoothed or geometric moving average chart. It evaluates the process level using an
  exponentially smoothed moving average. Here, by the term exponentially, we mean the procedure by which individual observations
  or subgroups are given progressively less importance or weight. When compared to the XChart, the EWMA chart is more sensitive
  to smaller shifts in the process level.<p>
  The exponentially weighted moving average is defined as<br>
  <code>
  XHat[t] = r*XHat[t] + (1-r)XHat[t-1] ,
  </code><br>
  where r is a constant and XHat[t] are EWMA chart points. The starting value for the first sample at time t = 1 = XHat[0] is
  grand mean value.
}
procedure EWMAChart(Data: TMtx; DrawVec: TVec; var CL, UCL, LCL: TSample; r: TSample = 0.3; Confidence: TSample = 0.997); overload;
{:@desc Calculates the Exponential Weighted Moving Average (EWMA) control chart. In this case UCL and LCL are not constant, but use an
  exact formula to calculate control limits for each point. It's worth noting that UCL and LCL values rapidly approach the asymptote value.
  @param Data Data of grouped responses. Each row contais a response at specific time.
    It's asumed the rows are in time order.
  @param r Weighting constant that weights past and current information. If,
    for example, r=0.3, 70% of the weight will be given to past information and
    30% to current information. Typically a r between 0.1 and 0.4 provides a
    reasonable balance between past and current information and 0.2 is very common in actual practice.
  @param Confidence
  @param DrawVec Returns the calculated EWMA chart points.
  @param CL Returns EWMA chart center line.
  @param LCL,UCL Returns EWMA chart lower and upper control limits.
    . In this case UCL and LCL are constant (asymptote) limits.
  @example  The following code will load data from file and create EWMA chart with
    r=0.25 and significance 95%:<p>
  <code>
  var CL: TSample;
    UCL, LCL: TVec;
    Data: TMtx;
    DrawVec: TVec;
    Confidence: TSample;
  begin
    Data := TMtx.Create;
    DrawVec := TVec.Create;
    UCL := TVec.Create;
    LCL := TVec.Create;
    try
      Data.LoadFromFile('ewma_data.mtx');
      EWMAChart(Data,DrawVec,CL,UCL,LCL,0.25,0.95);
      DrawValues(DrawVec,Series1);
      // Series2 and Series3 are line series used for displaying control limits
      DrawValues(UCL,Series2);
      DrawValues(LCL,Series3);
    finally
      Data.Destroy;
      DrawVec.Destroy;
      UCL.Destroy;
      LCL.Destroy;
    end;
  end;
  </code>
  @SeeAlso <See Class="TQCSeries">
}
procedure EWMAChart(Data: TMtx; DrawVec: TVec; var CL: TSample; UCL, LCL: TVec; r: TSample = 0.3; Confidence: TSample = 0.997); overload;

{:Constructs the Normal Probability Chart.
  Constructs the Normal Probability Chart. Use <See Class=TStatProbSeries> to visualize/plot constructed values.
  @param Data Data to be drawn.
  @param DataSorted If true, algorithm assumes Data is already sorted in ascending order.
    If Data is not sorted, you must set this parameter to false so that internal algorithm will
    automatically do the sorting.
  @param XDrawVec Retunrs vector of X values to be drawn.
  @param YDrawVec Returns vector of Y values to be drawn.
  @param minX,minY Returns slope line start point. These value are used by TStatProbSeries series.
  @param maxX,maxY Returns slope line end point. These value are used by TStatProbSeries series.
  @SeeAlso <See Class="TStatProbSeries">
  @example  The following code will create probability plot and then plot calculated values.<p>
  <code>
  var Series1: TStatProbSeries;
    Data, XVec, YVec: TVec;
    X1,Y1,X2,Y2: TSample;
  begin
    Data := TVec.Create;
    XVec := TVec.Create;
    YVec := TVec.Create;
    try
      // generate some random values for Data vec
      Data.Size(100);
      Data.RandGauss(0.0,1.0); // standard norm. dist.
      StatNormalPlot(Data,XVec,YVec,X1,X2,Y1,Y2,false);
      With Series1 do
      begin
        MinX := X1;
        MinY := Y1;
        MaxX := X2;
        MaxY := Y2;
      end;
      DrawValues(XVec,YVec,Series1);
    finally
      XVec.Destroy;
      YVec.Destroy;
      Data.Destroy;
    end;
  end;
  </code>
  @SeeAlso <See Class="TStatProbSeries">
  @SeeAlso <See Class="TStatAxisLabelTool">
  @cat Probabilities Plots
}
procedure StatNormalPlot(Data: TVec; XDrawVec, YDrawVec: TVec; var MinX, MaxX, MinY, MaxY: TSample; DataSorted: boolean = true);

{:Constructs the Quantile-Quantile Chart.
  Constructs the Quantile-Quantile Chart. Use <See Class=TStatProbSeries> to visualize/plot constructed values.
  @param XData X Data.
  @param YData Y Data.
  @param XDataSorted If true, algorithm assumes XData is already sorted in ascending order.
    If XData is not sorted, you must set this parameter to false so that internal algorithm will
    automatically do the sorting.
  @param YDataSorted If true, algorithm assumes YData is already sorted in ascending order.
    If YData is not sorted, you must set this parameter to false so that internal algorithm will
    automatically do the sorting.
  @param XDrawVec Retunrs vector of X values to be drawn.
  @param YDrawVec Returns vector of Y values to be drawn.
  @param minX,minY Returns slope line start point. These value are used by TStatProbSeries series.
  @param maxX,maxY Returns slope line end point. These value are used by TStatProbSeries series.
  @SeeAlso <See Class="TStatProbSeries">
  @example  The following code will create probability plot and then plot calculated values.<p>
  <code>
  var Series1: TStatProbSeries;
    XData, YData, XVec, YVec: TVec;
    X1,Y1,X2,Y2: TSample;
  begin
    XData := TVec.Create;
    YData := TVec.Create;
    XVec := TVec.Create;
    YVec := TVec.Create;
    try
      // generate some random values for Data vec
      XData.Size(100);
      YData.Size(100);
      XData.RandGauss(0.0,1.0); // standard norm. dist.
      YData.RandGauss(0.0,1.0); // standard norm. dist.
      // now construct QQ plot
      StatQQPlot(Data,XVec,YVec,X1,X2,Y1,Y2,false);
      With Series1 do
      begin
        MinX := X1;
        MinY := Y1;
        MaxX := X2;
        MaxY := Y2;
      end;
      DrawValues(XVec,YVec,Series1);
    finally
      XVec.Destroy;
      YVec.Destroy;
      Data.Destroy;
    end;
  end;
  </code>
  @SeeAlso <See Class="TStatProbSeries">
  @SeeAlso <See Class="TStatAxisLabelTool">
  @cat Probabilities Plots
}
procedure StatQQPlot(XData, YData: TVec; XDrawVec, YDrawVec: TVec; var MinX, MaxX, MinY, MaxY: TSample; XDataSorted: boolean = true; YDataSorted: boolean = true);

{:Constructs the Weibull Probability Chart.
  Constructs the Weibull Probability Chart. Use <See Class=TStatProbSeries> to visualize/plot
  constructed values.
  @param Data Data to be drawn.
  @param DataSorted If true, algorithm assumes Data is already sorted in ascending order.
    If Data is not sorted, you must set this parameter to false so that internal algorithm will
    automatically do the sorting.
  @param XDrawVec Retunrs vector of X values to be drawn.
  @param YDrawVec Returns vector of Y values to be drawn.
  @param minX,minY Returns slope line start point. These value are used by TStatProbSeries series.
  @param maxX,maxY Returns slope line end point. These value are used by TStatProbSeries series.
  @example
  <code>
  var Series1: TStatProbSeries;
    Data, XVec, YVec: TVec;
    X1,Y1,X2,Y2: TSample;
  begin
    Data := TVec.Create;
    XVec := TVec.Create;
    YVec := TVec.Create;
    try
      // generate some random values for Data vec
      Data.Size(100);
      RAndomWeibull(3,1.2,Data);
      StatWeibullPlot(Data,XVec,YVec,X1,X2,Y1,Y2,false);
      With Series1 do
      begin
        MinX := X1;
        MinY := Y1;
        MaxX := X2;
        MaxY := Y2;
      end;
      DrawValues(XVec,YVec,Series1);
    finally
      XVec.Destroy;
      YVec.Destroy;
      Data.Destroy;
    end;
  end;
  </code>
  @SeeAlso <See Class="TStatProbSeries">
  @SeeAlso <See Class="TStatAxisLabelTool">
  @cat Probabilities Plots
}
procedure StatWeibullPlot(Data: TVec; XDrawVec, YDrawVec: TVec; var MinX, MaxX, MinY, MaxY: TSample; DataSorted: boolean = true);

var
  {:@exclude}
  Tee_STATPROBSeries,
  {:@exclude}
  Tee_STATQCSeries,
  {:@exclude}
  Tee_DewGallery: String;

implementation

{$R StatToolsTee.res}  { series, tools icons }

procedure QCXChart(Data: TMtx; DrawVec: TVec; var CL, UCL, LCL: TSample; Confidence: TSample);
var M, c4, Delta: TSample;
begin
  M := NormalCDFInv(0.5+0.5*Confidence,0,1);
  c4 := Sqrt(2/(Data.Cols-1))*Gamma(Data.Cols*0.5)/Gamma((Data.Cols-1)*0.5);
 { evaluate sample standard deviation }
  StdRows(Data,DrawVec);
  Delta := M*DrawVec.Mean/(Sqrt(Data.Cols)*c4);
  { mean by group }
  Data.MeanRows(DrawVec);
  { grand mean }
  CL := DrawVec.Mean;
  UCL := CL + Delta;
  LCL := CL - Delta;
end;

procedure QCSChart(Data: TMtx; DrawVec: TVec; var CL, UCL, LCL: TSample; Confidence: TSample = 0.997);
var M, c4, Delta: TSample;
begin
  M := NormalCDFInv(0.5+0.5*Confidence,0,1);
  c4 := Sqrt(2/(Data.Cols-1))*Gamma(Data.Cols*0.5)/Gamma((Data.Cols-1)*0.5);
  StdRows(Data,DrawVec);
  Delta := M*Sqrt(1.0-Sqr(c4))*DrawVec.Mean/c4;
  { grand mean }
  CL := DrawVec.Mean;
  UCL := CL + Delta;
  LCL := CL - Delta;
  if LCL < 0 then LCL := 0;
end;

procedure QCRChart(Data: TMtx; DrawVec: TVec; var CL, UCL, LCL: TSample; Confidence: TSample);
var M, n, Delta: TSample;
begin
  M := NormalCDFInv(0.5+0.5*Confidence,0,1);
  n := Data.Cols;
  RangeRows(Data,DrawVec);
  { average range R }
  CL := DrawVec.Mean; { <R> }
  { For small n the result in more conservative control
    limits than the traditional d_2, d3 derivation
  }
  Delta := 0.5*M*CL*Sqrt(n*LN2/((n-1)*Ln(n)));
  UCL := CL + Delta;
  LCL := CL - Delta;
end;

procedure QCCapIndexes(Data: TVec; LB, UB: TSample; var p, CP, CPK : TSample;
                         var CPConfInt, CPKConfInt: TTwoElmReal; Alpha: TSample = 0.05);
var Mean, StdDev: TSample;
    n: Integer;
begin
  n := Data.Length;
  Mean := Data.Mean;
  StdDev := Data.StdDev(Mean);
  CP := NAN;
  CPK := NAN;
  if IsInf(LB) then p := 1.0 - NormalCDF(Lb,Mean,StdDev)
  else if IsInf(UB) then p := NormalCDF(Lb,Mean,StdDev)
  else
  begin
    p := 1.0 - NormalCDF(UB,Mean,StdDev) + NormalCDF(LB,Mean,StdDev);
    CPK := Min(Mean-LB,UB-Mean);
    CPK := CPK /(3*StdDev);
    CP := (UB-LB)/(6*StdDev);
    { CP confidence interval }
    CPConfInt[0] := CP*Sqrt(ChiSquareCDFInv(0.5*Alpha,n-1) /(n-1));
    CPConfInt[1] := CP*Sqrt(ChiSquareCDFInv(1.0 - 0.5*Alpha,n-1) /(n-1));
    { CPK confidence interval }
    CPKConfInt[0] := NormalCDFInv(1.0 - Alpha*0.5,0,1)*Sqrt((n-1)/(9*n*n-1) + Sqr(CPK)/(2*n-6)*(1.0+6/(n-1)));
    CPKConfInt[1] := CPK + CPKConfInt[0];
    CPKConfInt[0] := CPK - CPKConfInt[0];
  end;
end;

{ P Chart }
procedure QCPChart(Data: TVec; SampleSize : Integer; DrawVec: TVec; var CL, UCL, LCL : TSample; Confidence: TSample);
var M, DeltaInt : TSample;
begin
  M := NormalCDFInv(Confidence,0,1);
  DrawVec.Normalize(Data,0,SampleSize);
  CL := DrawVec.Mean;
  DeltaInt := M*Sqrt(CL*(1-CL)/SampleSize);
  LCL := CL - DeltaInt;
  UCL := CL + DeltaInt;
end;

procedure QCPChart(Data: TVec; SampleSize : TVec; DrawVec: TVec; var CL: TSample; UCL,LCL: TVec; Confidence: TSample);
const ProcName='QCPChart';
var M: TSample;
begin
  if (Data.Complex) or (SampleSize.Complex) then ERaise(ProcName+': (Data.Complex) or (SampleSize.Complex)');
  if Data.Length <> SampleSize.Length then ERaise(ProcName+': Data.Length <> SampleSize.Length');
  try
    M := NormalCDFInv(Confidence,0,1);
    DrawVec.DivideVec(Data,SampleSize);
    UCL.Size(DrawVec);
    LCL.Size(DrawVec);
    CL := Data.Sum / SampleSize.Sum;
    {  UCL[i] = CL + M*Sqrt((1-CL)*CL/n[i]) }
    {  LCL[i] = CL - M*Sqrt((1-CL)*CL/n[i]) }
    LCL.SetVal(CL);
    UCL.SetVal(CL*(1-CL)*M*M);
    UCL.DivideVec(SampleSize);
    UCL.Sqrt;
    LCL.Sub(UCL);
    UCL.Offset(-CL);
  except
    on E: Exception do ERaise(ProcName+' : '+E.Message);
  end;
end;

{ NP Chart }
procedure QCNPChart(Data: TVec; SampleSize : Integer; DrawVec: TVec; var CL, UCL, LCL : TSample; Confidence: TSample);
var M, DeltaInt : TSample;
    PAvg : TSample;
begin
  M := NormalCDFInv(Confidence,0,1);
  DrawVec.Copy(Data);
  CL := DrawVec.Mean;
  PAvg := CL / SampleSize;
  DeltaInt := M*Sqrt(PAvg*(1-PAvg)*SampleSize);
  LCL := CL - DeltaInt;
  UCL := CL + DeltaInt;
end;

procedure QCNPChart(Data: TVec; SampleSize : TVec; DrawVec: TVec; CL, UCL,LCL: TVec; Confidence: TSample);
const ProcName='QCPChart';
var M: TSample;
  tmp: TSample;
begin
  if (Data.Complex) or (SampleSize.Complex) then ERaise(ProcName+': (Data.Complex) or (SampleSize.Complex)');
  if Data.Length <> SampleSize.Length then ERaise(ProcName+': Data.Length <> SampleSize.Length');
  try
    M := NormalCDFInv(Confidence,0,1);
    CL.DivideVec(DrawVec,SampleSize);
    tmp := Data.Sum / SampleSize.Sum;
    CL.Copy(SampleSize);
    CL.Scale(tmp);
    LCL.Copy(CL);
    UCL.Sqrt(SampleSize);
    UCL.Scale(M*Sqrt(tmp*(1-tmp)));
    LCL.Sub(UCL);
    UCL.Add(CL);
  except
    on E: Exception do ERaise(ProcName+' : '+E.Message);
  end;
end;

{ U Chart }
procedure QCUChart(Data: TVec; SampleSize : Integer; DrawVec: TVec; var CL, UCL, LCL : TSample; Confidence: TSample);
var M, DeltaInt : TSample;
begin
  M := NormalCDFInv(Confidence,0,1);
  DrawVec.Normalize(Data,0,SampleSize);
  CL := DrawVec.Mean;
  DeltaInt := M*Sqrt(CL/SampleSize);
  LCL := CL - DeltaInt;
  UCL := CL + DeltaInt;
end;

procedure QCUChart(Data: TVec; SampleSize : TVec; DrawVec: TVec; var CL: TSample; UCL,LCL: TVec; Confidence: TSample);
const ProcName='QCPChart';
var M: TSample;
begin
  if (Data.Complex) or (SampleSize.Complex) then ERaise(ProcName+': (Data.Complex) or (SampleSize.Complex)');
  if Data.Length <> SampleSize.Length then ERaise(ProcName+': Data.Length <> SampleSize.Length');
  try
    M := NormalCDFInv(Confidence,0,1);
    DrawVec.DivideVec(Data,SampleSize);
    UCL.Size(DrawVec);
    LCL.Size(DrawVec);
    CL := Data.Sum / SampleSize.Sum;
    {  UCL[i] = CL + M*Sqrt(CL/n[i]) }
    {  LCL[i] = CL - M*Sqrt(CL/n[i]) }
    LCL.SetVal(CL);
    UCL.SetVal(CL*M*M);
    UCL.DivideVec(SampleSize);
    UCL.Sqrt;
    LCL.Sub(UCL);
    UCL.Offset(-CL);
  except
    on E: Exception do ERaise(ProcName+' : '+E.Message);
  end;
end;

{ C Chart }
procedure QCCChart(Data: TVec; DrawVec: TVec; var CL, UCL, LCL : TSample; Confidence: TSample);
var M, DeltaInt : TSample;
begin
  M := NormalCDFInv(Confidence,0,1);
  DrawVec.Copy(Data);
  CL := DrawVec.Mean;
  DeltaInt := M*Sqrt(CL);
  LCL := CL - DeltaInt;
  UCL := CL + DeltaInt;
end;

{ Normal Chart }
procedure StatNormalPlot(Data: TVec; XDrawVec, YDrawVec: TVec; var MinX, MaxX, MinY, MaxY: TSample; DataSorted: boolean);
const ProcName='StatNormalPlot';
var N: Integer;
  PX25,PX75, PY25, PY75,
  InvN,k,CX,CY : TSample;
begin
  if (Data.Complex) then ERaise(ProcName+': (Data.Complex)');
  if (Data.Length < 1 ) then ERaise(ProcName+': Data.Length < 1');
  N := Data.Length;
  invN := 1/N;
  XDrawVec.Copy(Data);
  YDrawVec.Size(XDrawVec);
  if Not(DataSorted) then XDrawVec.SortAscend;
  try
    { calculate min/max value for X }
    MinX := XDrawVec.Values[0];
    MaxX := XDrawVec.Values[N-1];
    PX25 := Percentile(XDrawVec,0.25);
    PX75 := Percentile(XDrawVec,0.75);
    YDrawVec.Ramp(0.5*invN,invN);
    NormalCDFInv(YDrawVec,0,1,YDrawVec);
    PY25 := Percentile(YDrawVec,0.25);
    PY75 := Percentile(YDrawVec,0.75);
    { calculate min/max value for Y }
    k := (PY75-PY25)/(PX75-PX25);
    CX := (PX25+PX75) * 0.5;
    CY := (PY25+PY75) * 0.5;
    MinY := CY +k*(minX-CX);
    MaxY := CY+k*(maxX-CX);
  except
    on E: Exception do ERaise(ProcName+' : '+E.Message);
  end;
end;

{ Quantile-Quantile Chart }
procedure StatQQPlot(XData, YData: TVec; XDrawVec, YDrawVec: TVec; var MinX, MaxX, MinY, MaxY: TSample; XDataSorted: boolean; YDataSorted: boolean);
const ProcName='StatQQPlot';
var N: Integer;
  PX25,PX75, PY25, PY75,
  k,CX,CY : TSample;
begin
  if (XData.Complex) or (YData.Complex) then ERaise(ProcName+': (XData.Complex) or YData.Complex');
  if (XData.Length <> YData.Length) then ERaise(ProcName+': (XData.Length <> YData.Length');
  if XData.Length < 1 then ERaise(ProcName+': XData.Length < 1');
  N := XData.Length;
  XDrawVec.Copy(XData);
  YDrawVec.Copy(YData);
  if Not(XDataSorted) then XDrawVec.SortAscend;
  if Not(YDataSorted) then YDrawVec.SortAscend;
  try
    { calculate min/max value for X }
    MinX := XDrawVec.Values[0];
    MaxX := XDrawVec.Values[N-1];
    PX25 := Percentile(XDrawVec,0.25);
    PX75 := Percentile(XDrawVec,0.75);
    PY25 := Percentile(YDrawVec,0.25);
    PY75 := Percentile(YDrawVec,0.75);
    { calculate min/max value for Y }
    k := (PY75-PY25)/(PX75-PX25);
    CX := (PX25+PX75) * 0.5;
    CY := (PY25+PY75) * 0.5;
    MinY := CY +k*(minX-CX);
    MaxY := CY+k*(maxX-CX);
  except
    on E: Exception do ERaise(ProcName+' : '+E.Message);
  end;
end;

{ Weibull Chart }
procedure StatWeibullPlot(Data: TVec; XDrawVec, YDrawVec: TVec; var MinX, MaxX, MinY, MaxY: TSample; DataSorted: boolean);
const ProcName='StatWeibullPlot';
var N: Integer;
  PX25,PX75, PY25, PY75,
  InvN,k: TSample;
begin
  if (Data.Complex) then ERaise(ProcName+': (Data.Complex)');
  if (Data.Length < 1 ) then ERaise(ProcName+': Data.Length < 1');
  N := Data.Length;
  invN := 1/N;
  XDrawVec.Copy(Data);
  YDrawVec.Size(XDrawVec);
  if Not(DataSorted) then XDrawVec.SortAscend;
  try
    { calculate min/max value for X }
    MinX := XDrawVec.Values[0];
    MaxX := XDrawVec.Values[N-1];
    PX25 := Ln(Percentile(XDrawVec,0.25));
    PX75 := Ln(Percentile(XDrawVec,0.75));
    YDrawVec.Ramp(1-0.5*invN,-invN);
    { Y = ln(ln(1/(1-Y)) }
    YDrawVec.Inv();
    YDrawVec.Ln();
    YDrawVec.Ln();
    PY25 := Percentile(YDrawVec,0.25);
    PY75 := Percentile(YDrawVec,0.75);
    { calculate min/max value for Y }
    k := (PY75-PY25)/(PX75-PX25);
    MinY := k*(Ln(minX)-PX25)+PY25;
    MaxY := k*(Ln(maxX)-PX25)+PY25;
  except
    on E: Exception do ERaise(ProcName+' : '+E.Message);
  end;
end;

{ Exponential moving average chart }
procedure EWMAChart(Data: TMtx; DrawVec: TVec; var CL, UCL, LCL: TSample; r: TSample; Confidence: TSample);
const ProcName='EWMAChart';
var XBar: TVec;
    c4,mult: TSample;
    Delta: TSample;
    i,m,n: Integer;
    dvp,xbarp: TSampleArray;
begin
  if (Data.Complex) then ERaise(ProcName+': (Data.Complex)');
  m := Data.Rows;
  n := Data.Cols;
  try
    CreateIt(XBar);
    try
      Data.MeanRows(XBar);
      CL := XBar.Mean;
      DrawVec.Size(m,false);
      Enlist(DrawVec,XBar,dvp,xbarp);
      dvp[0] := r*xbarp[0]+(1-r)*CL;
      for i := 1 to DrawVec.Length -1 do
        dvp[i] := r*xbarp[i] +(1-r)*dvp[i-1];
      Dismiss(dvp,xbarp);
      if n > 1 then
      begin
        StdRows(Data,XBar);
        mult := NormalCDFInv(Confidence,0,1);
        c4 := Sqrt(2/(n-1))*Gamma(0.5*n)/Gamma((n-1)*0.5);
        Delta := mult*XBar.Mean/(c4)*Sqrt(r/((2-r)*n));
        UCL := CL + Delta;
        LCL := CL - Delta;
      end else
      begin
        UCL := INF;
        LCL := -INF;
      end;
    finally
      FreeIt(XBar);
    end;
  except
    on E: Exception do ERaise(ProcName+' : '+E.Message);
  end;
end;

procedure EWMAChart(Data: TMtx; DrawVec: TVec; var CL: TSample; UCL, LCL: TVec; r: TSample = 0.3; Confidence: TSample = 0.997); overload;
const ProcName='EWMAChart';
var XBar: TVec;
    Delta,mult,c4: TSample;
    i,m,n: Integer;
    dvp,xbarp: TSampleArray;
begin
  if (Data.Complex) then ERaise(ProcName+': (Data.Complex)');
  m := Data.Rows;
  n := Data.Cols;
  try
    CreateIt(XBar);
    try
      Data.MeanRows(XBar);
      CL := XBar.Mean;
      DrawVec.Size(m,false);
      UCL.Size(m,false);
      LCL.Size(m,false);
      Enlist(DrawVec,XBar,dvp,xbarp);
      dvp[0] := r*xbarp[0]+(1-r)*CL;
      for i := 1 to DrawVec.Length -1 do
        dvp[i] := r*xbarp[i] +(1-r)*dvp[i-1];
      Dismiss(dvp,xbarp);
      if n> 1 then
      begin
        // now get the ucl and lcl for each point
        mult := NormalCDFInv(Confidence,0,1);
        c4 := Sqrt(2/(n-1))*Gamma(0.5*n)/Gamma((n-1)*0.5);
        StdRows(Data,XBar);
        Delta := mult*XBar.Mean*Sqrt(1/n)/c4;
        Enlist(UCL,LCL,dvp,xbarp);
        for i := 1 to m do
        begin
          // UCL
          dvp[i-1] := cl + Delta*Sqrt(r/(2.0-r)*(1-Power(1-r,2*i)));
          // LCL
          xbarp[i-1] := cl - Delta*Sqrt(r/(2.0-r)*(1-Power(1-r,2*i)));
        end;
        Dismiss(dvp,xbarp);
      end else
      begin
        UCL.SetVal(INF);
        LCL.SetVal(-INF);
      end;
    finally
      FreeIt(XBar);
    end;
  except
    on E: Exception do ERaise(ProcName+' : '+E.Message);
  end;
end;




{ TStatProbSeries }
procedure TStatProbSeries.AddSampleValues(NumValues: Integer);
begin
  inherited;
  MinX:= 0;
  MaxX := Self.XValues.Count;
  MinY := Self.YValues.MinValue;
  MaxY := Self.YValues.MaxValue;
end;

constructor TStatProbSeries.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSlopePen := CreateChartPen;
  FSlopePen.Style := psDot;
end;

destructor TStatProbSeries.Destroy;
begin
  FSlopePen.Destroy;
  inherited Destroy;
end;

procedure TStatProbSeries.DoAfterDrawValues;
var P1,P2: TPoint;
begin
  if (FSlopePen.Visible) then
  begin
    P1.x := CalcXPosValue(minX);
    P1.y := CalcYPosValue(minY);
    P2.x := CalcXPosValue(maxX);
    P2.y := CalcYPosValue(maxY);
    With ParentChart,ParentChart.Canvas do
    begin
      Brush.Style := bsClear;
      AssignVisiblePen(FSlopePen);
      ClipRectangle(ChartRect);
      MoveTo(P1.x,P1.y);
      LineTo(P2.x,P2.y);
      UnClipRectangle;
    end;
  end;
  inherited DoAfterDrawValues;
end;

procedure TStatProbSeries.SetSeriesColor(AColor: TColor);
begin
  inherited SetSeriesColor(AColor);
  FSlopePen.Color := AColor;
end;

procedure TStatProbSeries.SetSlopePen(const Value: TChartPen);
begin
  FSlopePen.Assign(Value);
end;

{ TSPCSeries }
procedure TQCSeries.Assign(Source: TPersistent);
begin
  if Source is TQCSeries then
  begin
    Self.FLCL := TQCSeries(Source).FLCL;
    Self.FUCL := TQCSeries(Source).FUCL;
    Self.FCL := TQCSeries(Source).FCL;
    Self.FShowText := TQCSeries(Source).FShowText;
    Self.Pointer := TQCSeries(Source).Pointer;
    Self.CLPen := TQCSeries(Source).FCLPen;
    Self.ControlLimitPen := TQCSeries(Source).FControlLimitPen;
  end;
  inherited;
  Self.Pointer.Visible := TQCSeries(Source).Pointer.Visible; { trick for TLineSeries }
end;

constructor TQCSeries.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Pointer.Visible := true;
  FShowText := True;
  FControlLimitPen := CreateChartPen;
  FCLPen := CreateChartPen;
  FControlLimitPen.Style := psDot;
end;

destructor TQCSeries.Destroy;
begin
  FControlLimitPen.Free;
  FCLPen.Free;
  inherited Destroy;
end;

procedure TQCSeries.DoAfterDrawValues;
var tmpY : Integer;
    tH : Integer;
begin
  inherited DoAfterDrawValues;
  With ParentChart, ParentChart.Canvas do
  begin
    AssignFont(Self.Marks.Font);
    tH := TextHeight('W');
    Brush.Style := bsClear;
    ClipRectangle(ChartRect);
    { draw upper-lower Control Limit }
    AssignVisiblePen(FControlLimitPen);
    tmpY := CalcYPosValue(UCL);
    MoveTo(ChartRect.Left, tmpY);
    LineTo(ChartRect.Right, tmpY);
    if FShowText then TextOut(ChartRect.Left+50,tmpY,'UCL = '+FormatFloat('0.00',FUCL));
    tmpY := CalcYPosValue(LCL);
    MoveTo(ChartRect.Left, tmpY);
    LineTo(ChartRect.Right, tmpY);
    if FShowText then TextOut(ChartRect.Left+50,tmpY-tH-FControlLimitPen.Width,'LCL = '+FormatFloat('0.00',FLCL));

    { draw average (mean or StdDev}
    AssignVisiblePen(FCLPen);
    tmpY := CalcYPosValue(FCL);
    MoveTo(ChartRect.Left, tmpY);
    LineTo(ChartRect.Right, tmpY);
    if FShowText then TextOut(ChartRect.Left+50,tmpY,'CL = '+FormatFloat('0.00',FCL));
    UnclipRectangle;
  end;
end;

class function TQCSeries.GetEditorClass: String;
begin
  result:='TQCSeriesEditor';
end;

function TQCSeries.IsValidSourceOf(Value: TChartSeries): Boolean;
begin
  result := Value is TQCSeries;
end;

function TQCSeries.MaxYValue: Double;
begin
  Result := Max(inherited MaxYValue,FUCL)
end;

function TQCSeries.MinYValue: Double;
begin
  Result := Min(inherited MinYValue,FLCL)
end;

procedure TQCSeries.SetCL(const Value: TSample);
begin
  if FCL <> Value then
  begin
    FCL := Value;
    Repaint;
  end;
end;

procedure TQCSeries.SetCLPen(const Value: TChartPen);
begin
  FCLPen.Assign(Value);
end;

procedure TQCSeries.SetControlLimitPen(const Value: TChartPen);
begin
  FControlLimitPen.Assign(Value);
end;

procedure TQCSeries.SetLCL(const Value: TSample);
begin
  if FLCL <> Value then
  begin
    FLCL := Value;
    Repaint;
  end;
end;

procedure TQCSeries.SetSeriesColor(AColor: TColor);
begin
  inherited SetSeriesColor(AColor);
  FCLPen.Color := AColor;
  FControlLimitPen.Color := AColor;
end;

procedure TQCSeries.SetShowText(const Value: boolean);
begin
  if FShowText<> Value then
  begin
    FShowText := Value;
    Repaint;
  end;
end;

procedure TQCSeries.SetUCL(const Value: TSample);
begin
  if FUCL <> Value then
  begin
    FUCL := Value;
    Repaint;
  end;
end;

initialization
  Tee_DewGallery := 'Dew Research';
  Tee_STATPROBSeries := 'Probability';
  Tee_STATQCSeries := 'Quality Control';
  RegisterTeeSeries(TStatProbSeries,@Tee_STATPROBSeries,@Tee_DewGallery,1);
  RegisterTeeSeries(TQCSeries,@Tee_STATQCSeries,@Tee_DewGallery,1);

  finalization
  UnRegisterTeeSeries( [TStatProbSeries,TQCSeries]);

end.
